#! /usr/bin/perl -w

use strict;

# Picks and returns a random element of the supplied list
sub random_of (@) {
    $_[int rand @_]
}

# Picks a random path from a {...|...} markuped string
sub random_path ($) {
    my ($set) = @_;

    while ($set =~ s/\{([^{}]*\|[^{}]*)\}/random_of split m[\|], $1/ge) {}
    return $set;
}

sub make_random_string ($) {
    my ($filename) = @_;

    # Read the lines
    my $fh;
    open $fh, "<$filename" or die "$filename: open: $!";
    my @lines = <$fh>;
    close $fh;
    chomp for @lines;

    # Pick a random line and a random path from it
    my $result = random_path random_of @lines;

    # Return
    return $result;
}

# Replaces @FOO@ in $string with $subst{FOO} or, if it's a sub ref, with
# &{$subst{FOO}}($string).
sub atexpand ($%) {
    my ($string, %subst) = @_;
    my @parts = split /\@/, $string, -1;
    return '' unless @parts; # avoid OOB problems later
    my $result = $parts[0];
    my $i;
    for ($i = 1; $i < $#parts; $i++) {
        my $p = $parts[$i];
        if (exists $subst{$p}) {
            my $repl = $subst{$p};
            $repl = &$repl($p) if ref $repl eq 'CODE';
            $result .= $repl.$parts[++$i];
        } else {
            $result .= "\@$p";
        }
    }
    $result .= "\@$parts[$#parts]" if $i == $#parts;
    return $result;
}

my $filename = shift @ARGV;
die "No filename specified"
    unless defined $filename;

my %subst = @ARGV;
print atexpand(make_random_string($filename), %subst), "\n";
